"use strict";

var Helper = require('./helper');

require("./mocha-config.js");

var Alias = require('../../main/js/map/data/Alias');
var minerva = require('../../main/js/minerva');
var Project = require('../../main/js/map/data/Project');
var GuiConnectorMock = require('./GuiConnector-mock');
var ServerConnectorMock = require('./ServerConnector-mock');
var ServerConnectorMock = require('./ServerConnector-mock');

var chai = require('chai');
var assert = chai.assert;
var logger = require('./logger');

describe('minerva global', function() {
  beforeEach(function() {
    global.scriptDiv = document.createElement("script");
    global.GuiConnector = undefined;
    global.ServerConnector = undefined;
    document.body.appendChild(global.scriptDiv);
  });

  afterEach(function() {
    document.body.removeChild(global.scriptDiv);
    global.GuiConnector = GuiConnectorMock;
    global.ServerConnector = ServerConnectorMock;
  });

  describe('create', function() {
    it('default', function() {
      var options = null;
      return ServerConnectorMock.getProject().then(function(project) {
        options = helper.createCustomMapOptions(project);

        return minerva.create(options);
      }).then(function(result) {
        assert.ok(result);
        assert.equal(logger.getWarnings().length, 0);
      });
    });
    it('invalid projectId', function() {
      var options = {
        projectId : "unknownId",
        element : testDiv,
      };
      return minerva.create(options).then(function() {
        assert.ok(false);
      }, function(error) {
        assert.ok(error.message.indexOf("Project with given id doesn't exist") >= 0);
      });
    });
  });

  it('create with overview', function() {
    return ServerConnectorMock.readFile("testFiles/projectWithImages.json").then(function(fileContent) {
      var project = new Project(JSON.parse(fileContent));
      var options = helper.createOptions(project);

      return minerva.create(options);
    }).then(function(result) {
      assert.ok(result);
      assert.equal(logger.getWarnings().length, 0);
      var showOverviewButton = document.getElementsByName("showOverviewButton")[0];
      showOverviewButton.onclick();
    });
  });

  it("showComments", function() {
    var options = null;
    return ServerConnectorMock.getProject().then(function(project) {
      options = helper.createCustomMapOptions(project);
      return minerva.create(options);
    }).then(function() {
      var commentCheckbox = document.getElementsByName("commentCheckbox")[0];
      commentCheckbox.checked = true;
      return commentCheckbox.onclick();
    }).then(function() {
      assert.ok(ServerConnectorMock.getSessionData(options.getProject()).getShowComments());
    });
  });

  it("contructor with GET zoom param", function() {
    helper.setUrl("http://test/?zoom=5");
    var options;
    return ServerConnectorMock.getProject().then(function(project) {
      options = helper.createCustomMapOptions(project);
      return minerva.create(options);
    }).then(function() {
      var sessionData = ServerConnectorMock.getSessionData(options.getProject());
      assert.equal(sessionData.getZoomLevel(options.getProject().getModel()), 5);
    });
  });

  it("contructor with GET coord param", function() {
    helper.setUrl("http://test/?x=5&y=6");
    var options;
    return ServerConnectorMock.getProject().then(function(project) {
      options = helper.createCustomMapOptions(project);
      return minerva.create(options);
    }).then(function() {
      var center = ServerConnectorMock.getSessionData(options.getProject()).getCenter(options.getProject().getModel());
      assert.ok(center instanceof google.maps.Point);
      assert.equal(center.x, 5);
      assert.equal(center.y, 6);
    });
  });

  it('create with layout', function() {
    var layout;
    var project;
    return ServerConnectorMock.getProject().then(function(result) {
      project = result;
      var options = helper.createCustomMapOptions(project);

      layout = project.getModel().getLayouts()[1];

      helper.setUrl("http://test/?layout=" + layout.getName());

      return minerva.create(options);
    }).then(function(result) {
      assert.ok(result);
      // input file is not available so it's the background
      return result.getVisibleDataOverlays();
    }).then(function(visibleDataOverlays) {
      // input file is available so it's not the background file but overlay
      assert.equal(visibleDataOverlays.length, 0);
      assert.equal(ServerConnectorMock.getSessionData(project).getSelectedBackgroundOverlay(), layout.getId());
      assert.equal(logger.getWarnings().length, 0);
    });
  });

  it('create with layout from session data', function() {
    var layout;
    return ServerConnectorMock.getProject().then(function(project) {
      var options = helper.createCustomMapOptions(project);

      layout = project.getModel().getLayouts()[1];

      ServerConnectorMock.getSessionData(project).setSelectedBackgroundOverlay(layout.getId());

      return minerva.create(options);
    }).then(function() {
      assert.equal(ServerConnectorMock.getSessionData().getSelectedBackgroundOverlay(), layout.getId());
    });
  });

  it('create with layout 2', function() {
    var layout = helper.createLayout();
    layout.setInputDataAvailable(true);
    // disable upload of the data from server
    layout.setInitialized(true);
    var layoutName = layout.getName();

    helper.setUrl("http://test/?layout=" + layoutName);

    return ServerConnectorMock.getProject().then(function(project) {
      project.getModel().addLayout(layout);

      var options = helper.createCustomMapOptions(project);
      return minerva.create(options);
    }).then(function(result) {
      assert.ok(result);
      return result.getVisibleDataOverlays();
    }).then(function(visibleDataOverlays) {
      // input file is available so it's not the background file but overlay
      assert.equal(visibleDataOverlays.length, 1);
      assert.equal(logger.getWarnings().length, 0);
    });
  });

  it('create with search overlay and GET search param', function() {
    helper.setUrl("http://test/?search=s1");

    return ServerConnectorMock.getProject().then(function(project) {
      var options = helper.createCustomMapOptions(project);
      return minerva.create(options);
    }).then(function(result) {
      return result.getHighlightedBioEntities("search");
    }).then(function(elements) {
      assert.ok(elements.length > 0);
    });
  });

  it('create with show sumbodel GET param', function() {
    helper.setUrl("http://test/?submap=15781");

    var project = helper.createProject();
    project.getModel().setId(15781);

    return ServerConnectorMock.getProject().then(function(project) {
      var options = helper.createCustomMapOptions(project);

      return minerva.create(options);
    });
  });

  it("contructor with touch enabled", function() {
    return ServerConnectorMock.getProject().then(function(project) {
      var options = helper.createCustomMapOptions(project);
      options.setCustomTouchInterface(true);
      return minerva.create(options).then(function(result) {
        assert.ok(result);
      });
    });
  });

  it('add search listener', function() {
    var callbackOk = false;
    var globalResult;
    return ServerConnectorMock.getProject().then(function(project) {
      var options = helper.createCustomMapOptions(project);
      options.setDebug(true);
      return minerva.create(options);
    }).then(function(result) {
      globalResult = result;

      result.addListener({
        dbOverlayName : "search",
        type : "onSearch",
        callback : function(elements) {
          assert.ok(elements.length > 0);
          assert.ok(elements[0].length !== undefined, "Array of arrays expected as onSearch result");
          callbackOk = true;
        }
      });
      return result.customMap.getOverlayByName("search").searchByQuery("s1");
    }).then(function() {
      assert.ok(callbackOk);
      callbackOk = false;
      return globalResult.customMap.getOverlayByName("search").searchByQuery("s1");
    }).then(function() {
      assert.ok(callbackOk);
      callbackOk = false;
      var params = {
        coordinates : new google.maps.Point(184.79, 365.76),
        zoom : 2,
        modelId : globalResult.getProject().getModel().getId(),
      };
      return globalResult.customMap.getOverlayByName("search").searchByCoordinates(params);
    }).then(function() {
      assert.ok(callbackOk);
    });
  });

  it('getProject', function() {
    return ServerConnectorMock.getProject().then(function(project) {
      var options = helper.createCustomMapOptions(project);
      return minerva.create(options);
    }).then(function(result) {
      assert.equal(result.getProject().getProjectId(), "sample");
    });
  });

  it("simple contructor", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      assert.ok(result);
    }).then(function() {
      globalResult.destroy();
    });
  });

  it("getBioEntityById", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return result.getBioEntityById({
        id : 329177,
        modelId : 15781,
        type : "ALIAS"
      });
    }).then(function(result) {
      assert.ok(result);
    }).then(function() {
      globalResult.destroy();
    });
  });

  it("getBioEntityById for reaction", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return result.getBioEntityById({
        id : 153508,
        modelId : 15781,
        type : "REACTION"
      });
    }).then(function(result) {
      assert.ok(result);
      assert.ok(result.getReactants()[0] instanceof Alias);
    }).then(function() {
      globalResult.destroy();
    });
  });

  it("showElement", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    var elementToShow = {
      element : {
        id : 329171,
        modelId : 15781,
        type : "ALIAS"
      },
      type : "ICON",
    };

    var elementToShow2 = {
      element : {
        id : 329171,
        modelId : 15781,
        type : "ALIAS"
      },
      type : "SURFACE",
      options : {
        color : "#FF0000",
      }
    };
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return globalResult.showBioEntity(elementToShow);
    }).then(function() {
      return globalResult.getHighlightedBioEntities();
    }).then(function(elements) {
      assert.equal(elements.length, 1);
      return globalResult.showBioEntity(elementToShow2);
    }).then(function() {
      return globalResult.getHighlightedBioEntities();
    }).then(function(elements) {
      assert.equal(elements.length, 2);
      return globalResult.hideBioEntity(elementToShow);
    }).then(function() {
      return globalResult.getHighlightedBioEntities();
    }).then(function(elements) {
      assert.equal(elements.length, 1);
      return globalResult.hideBioEntity(elementToShow2);
    }).then(function() {
      return globalResult.getHighlightedBioEntities();
    }).then(function(elements) {
      assert.equal(elements.length, 0);
      globalResult.destroy();
    });
  });

  it("showElement twice", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    var elementToShow = {
      element : {
        id : 329171,
        modelId : 15781,
        type : "ALIAS"
      },
      type : "ICON",
    };
    var elementToShow2 = {
      element : {
        id : 329171,
        modelId : 15781,
        type : "ALIAS"
      },
      type : "SURFACE",
      options : {
        color : "#FF0000",
      }
    };

    return minerva.create(options).then(function(result) {
      globalResult = result;
      globalResult.showBioEntity(elementToShow);
      return globalResult.showBioEntity(elementToShow2);
    }).then(function() {
      globalResult.destroy();
      assert.ok(false, "Should be rejected due to fact that there is show in progress");
    }, function(error) {
      assert.ok(error.message.indexOf("wait until previous Promise for showBioEntity/hideBioEntity is resolved") >= 0);
      globalResult.destroy();
    });
  });

  it("setCenter", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return result.setCenter({
        modelId : 15781,
        x : 10,
        y : 20,
      });
    }).then(function() {
      var sessionData = ServerConnectorMock.getSessionData(globalResult.getProject());
      var center = sessionData.getCenter(globalResult.getProject().getModel());
      assert.ok(center instanceof google.maps.Point);
      assert.closeTo(parseFloat(center.x), 10, Helper.EPSILON);
      assert.closeTo(parseFloat(center.y), 20, Helper.EPSILON);
    }).then(function() {
      globalResult.destroy();
    });
  });

  it("setZoom", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return result.setZoom({
        modelId : 15781,
        zoom : 10,
      });
    }).then(function() {
      var sessionData = ServerConnectorMock.getSessionData(globalResult.getProject());
      var zoom = sessionData.getZoomLevel(globalResult.getProject().getModel());
      assert.equal(zoom, 10);
    }).then(function() {
      globalResult.destroy();
    });
  });

  it("getConfiguration", function() {
    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return result.getConfiguration();
    }).then(function(conf) {
      assert.ok(conf.getElementTypes().length > 0);
    }).then(function() {
      globalResult.destroy();
    });
  });

  it("getReactionByParticipantId", function() {
    var ie = {
      modelId : 15781,
      type : "ALIAS",
      id : 329167
    };

    var options = {
      projectId : "sample",
      element : testDiv
    };
    var globalResult;
    return minerva.create(options).then(function(result) {
      globalResult = result;
      return result.getReactionsWithElement(ie);
    }).then(function(reactions) {
      assert.equal(reactions.length, 5);
    }).then(function() {
      globalResult.destroy();
    });

  });

  it('create Export', function() {
    var options = null;
    return ServerConnectorMock.getProject().then(function(project) {
      options = helper.createCustomMapOptions(project);

      return minerva.createExport(options);
    }).then(function(result) {
      assert.ok(result);
      assert.equal(logger.getWarnings().length, 0);
    });
  });

});
